
DATA_TYPES = {
    'integer': 'Integer',
    'float': 'Float',
    'datetime': 'Date',
    'boolean': 'Boolean',
    'string': 'Text',
    'file': 'File'
}

SCHEMA_OBJS = {
    'entity': 'Entities',
    'relation': 'Relations'
}
