# This file exists because I'm not sure we'll need to subclass them later.
from werkzeug.exceptions import HTTPException, NotFound, Gone
from werkzeug.exceptions import BadRequest, Unauthorized, Forbidden
from werkzeug.exceptions import InternalServerError as InternalError
from werkzeug.exceptions import NotImplemented
