"""empty message

Revision ID: 5a34d3b6a160
Revises: 3a3c75de71b
Create Date: 2014-03-14 15:51:15.830107

"""

# revision identifiers, used by Alembic.
revision = '5a34d3b6a160'
down_revision = '3a3c75de71b'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('grano_property', sa.Column('value_boolean', sa.Boolean(), nullable=True))
    op.add_column('grano_property', sa.Column('value_datetime', sa.DateTime(), nullable=True))
    op.add_column('grano_property', sa.Column('value_float', sa.Float(), nullable=True))
    op.add_column('grano_property', sa.Column('value_integer', sa.Integer(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('grano_property', 'value_integer')
    op.drop_column('grano_property', 'value_float')
    op.drop_column('grano_property', 'value_datetime')
    op.drop_column('grano_property', 'value_boolean')
    ### end Alembic commands ###
